#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>
#include <strings.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/SoPath.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <Xm/Label.h>

#include <Inventor/SoDB.h>

#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/SoOutput.h>


SoSeparator *SceneRoot;



void
printVec( SbVec3f vec , char *s){

printf("%s       (%22.18f %22.18f %22.18f)\n", s, vec[0], vec[1], vec[2] );

}


SoPath *
PickSelectionCallback(void *root, const SoPickedPoint *pick)
{
  SbVec3f normal_vector;
  SbVec3f point_vector;
  
  SbVec3f normalObj;
  SbVec3f pointObj;
    
  // Get Normal of picked face
  normal_vector = pick->getNormal();
  point_vector  = pick->getPoint();
  
  normalObj = pick->getObjectNormal();
  pointObj  = pick->getObjectPoint();

  printVec( normal_vector, "-->SELECTION>>\n\n.Normal vector (WORLD)::");
  printVec( normalObj, ".Normal vector (OBJECT)::");
  printVec( point_vector, ">Point(World)");
  printVec( pointObj, ">Point(Obj)");

  SoSphere *smallsphere=new SoSphere;
  smallsphere->radius.setValue(0.1);
  SoSeparator *sphere=new SoSeparator;
  SceneRoot->addChild(sphere);
  SoTranslation *trans=new SoTranslation;
  trans->translation.setValue(pointObj);
  SoBaseColor *color=new SoBaseColor;
  color->rgb.setValue(0.0,0.0,1.0);
  sphere->addChild(color);
  sphere->addChild(trans);
  sphere->addChild(smallsphere);
  return NULL;

}

void
SelectionCallback( void *root, SoPath *p ){
}
void
DeSelectionCallback( void *, SoPath *p ){
}


void
init_scene()
{
    SceneRoot=new SoSeparator;
    SceneRoot->ref();    SceneRoot->ref();
}


int
addAnIvFile( char *fname)
{
  SoInput myInput;

  if (!myInput.openFile(fname)) {
    printf("Cannot open %s\n", fname);
    return -1;
  }

  SoSeparator *fileContents = (SoDB::readAll(&myInput));
  if (fileContents != NULL){
    SoSelection *selNode=new SoSelection;

    selNode->policy = SoSelection::SINGLE;
    selNode->setPickFilterCallback(PickSelectionCallback, fileContents);
    selNode->addSelectionCallback(SelectionCallback, selNode);
    selNode->addDeselectionCallback(DeSelectionCallback, NULL);


    SceneRoot->addChild(selNode);
    selNode->addChild(fileContents);


    return 0;
  } else {
      printf("IV FILE is not an IV file? \n");
      return -2;
  }






}



void print_usage(char *pn){
  printf("%s inventor-file\n", pn);
}








int
main(int argc, char **argv)
{

  char IVNAME[255];
  if (argc==1) {
    print_usage(argv[0]);
    return 0;
  } else strcpy(IVNAME, argv[1]);
  
 
  init_scene();



  Widget altWindow;
  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow  == NULL) exit(1);

  SoXtExaminerViewer *MainExaminerViewer=new SoXtExaminerViewer( myWindow );
  MainExaminerViewer->setSceneGraph( SceneRoot );
  MainExaminerViewer->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_BLEND);
  MainExaminerViewer->setTitle(argv[0]);


  MainExaminerViewer->show();
  XtRealizeWidget(myWindow);


  addAnIvFile(IVNAME);

  
  SoXt::mainLoop();  
}




